import java.util.Arrays;
import java.util.Random;

/**
   This program measures the runtime of the bubble
   sort algorithm on a sequence of growing arrays.

   In this version of the profiler we measure both the
   amount of time it takes to do a sort and we also
   measure the number of comparisons and swaps that
   take place during a sort.

   For each size n of the array, we sort three kinds
   of arrays, an already sorted array, a random array,
   and an array that is sorted backwards. The first array,
   since it is already sorted, should be the best case. The
   second array, since it is random, should be the average
   case. And the third array, since it is the opposite of
   being sorted, should be the worst case.

   The bubble sort algorithm is rewritten here in a slightly
   different way to make it easier to count the comparisons and
   swaps.
*/
public class BubbleSort_Profile_v2
{
   private static long cnt1 = 0; // count comparisons
   private static long cnt2 = 0; // count swaps

   public static void main(String[] args)
   {
      final int size = 10_000;   // starting size for the arrays
      final int iterations = 12; // how many times to increase the array size

      System.out.println("Bubble sort.");

      for (int i = 1; i <= iterations; ++i)
      {
         int[] array = new int[i*size];
         for (int j = 0; j < array.length; ++j)
         {
            array[j] = j;
         }
         long startTime = System.currentTimeMillis();
            sort(array);
         long stopTime = System.currentTimeMillis();
         final long bestCase =(stopTime - startTime);
         final long bestCount1 = cnt1;
         final long bestCount2 = cnt2;

         array = BubbleSort.randomIntArray(i*size, 1_000_000);
         startTime = System.currentTimeMillis();
            sort(array);
         stopTime = System.currentTimeMillis();
         final long avgCase =(stopTime - startTime);
         final long avgCount1 = cnt1;
         final long avgCount2 = cnt2;

         array = new int[i*size];
         for (int j = 0; j < array.length; ++j)
         {
            array[j] = i*size - j;
         }
         startTime = System.currentTimeMillis();
            sort(array);
         stopTime = System.currentTimeMillis();
         final long worstCase =(stopTime - startTime);
         final long worstCount1 = cnt1;
         final long worstCount2 = cnt2;

         System.out.println(
            "For n = " + (i*size) + ", "
          + "best time = " + bestCase + " (" + bestCount1 + ", " + bestCount2 + "), "
          + "avg time = " + avgCase + " (" + avgCount1 + ", " + avgCount2 + "), "
          + "worst time = " + worstCase + " (" + worstCount1 + ", " + worstCount2 + ")");
      }
   }

   /**
      Sort an array using iterative bubble sort.

      @param a  the array to sort
   */
   public static void sort(int[] a)
   {
      cnt1 = 0; cnt2 = 0;

      for (int i = 0; i < a.length - 1; ++i)
      {
         for (int j = 0; j < a.length - 1 - i; ++j)
         {
            cnt1++; // count comparisons
            if ( a[j] > a[j + 1] )
            {
               cnt2++; // count swaps
               swap(a, j, j + 1);
            }
         }
      }
   }


   /**
      Swap two entries of an array.

      @param a  the array
      @param i  the first position to swap
      @param j  the second position to swap
   */
   public static void swap(int[] a, int i, int j)
   {
      int temp = a[i];
      a[i] = a[j];
      a[j] = temp;
   }
}
